import {AccessChangeQueryParameters, AccessSearch, AccessSearchFail, AccessSearchSuccess} from "@app/access/access.action";
import {CrudStateModel} from "@app/shared/crud.state";
import {AccessResourceApiEnum} from "@app/shared/enums/api.enum";
import {StateEnum} from "@app/shared/enums/state.enum";
import {ArchiveMetadataModel} from "@app/shared/models/business/archive-metadata.model";
import {CollectionTypedModel} from "@app/shared/models/collection-typed.model";
import {QueryParametersModel} from "@app/shared/models/query-parameters.model";
import {ApiService} from "@app/shared/services/api.service";
import {CrudHelperUtil} from "@app/shared/utils/crud-helper.util";
import {MapUtil} from "@app/shared/utils/map.util";
import {StringUtil} from "@app/shared/utils/string.util";
import {Action, State, StateContext} from "@ngxs/store";
import {tap} from "rxjs/internal/operators/tap";
import {catchError} from "rxjs/operators";

export interface AccessStateModel extends CrudStateModel<ArchiveMetadataModel> {
  search: string;
}

@State<AccessStateModel>({
  name: StateEnum.access,
  defaults: {
    search: null,
    total: 0,
    isLoading: false,
    list: [],
    current: null,
    queryParameters: new QueryParametersModel(),
  },
})
export class AccessState {
  constructor(private apiService: ApiService) {
  }

  private readonly DEFAULT_SEARCH = "*";

  @Action(AccessChangeQueryParameters)
  changeQueryParameters(ctx: StateContext<AccessStateModel>, action: AccessChangeQueryParameters): void {
    ctx.patchState({
      queryParameters: action.queryParameters,
    });
    ctx.dispatch(new AccessSearch(ctx.getState().search));
  }

  @Action(AccessSearch, {cancelUncompleted: true})
  searchDip(ctx: StateContext<AccessStateModel>, action: AccessSearch) {
    const searchString = this.getSearchStringToApply(ctx, action);
    ctx.patchState({
      isLoading: true,
      search: searchString,
    });

    const pathFilter = "query";
    const queryParameters = CrudHelperUtil.getQueryParametersToApply(action.queryParameters, ctx);
    MapUtil.addOrUpdate(queryParameters.search.searchItems, pathFilter, searchString);

    if (searchString === StringUtil.stringEmpty) {
      ctx.dispatch(new AccessSearchSuccess(null));
    } else {
      return this.apiService.get<ArchiveMetadataModel>(AccessResourceApiEnum.metadataSearch, queryParameters)
        .pipe(
          tap((collection: CollectionTypedModel<ArchiveMetadataModel>) => {
            ctx.dispatch(new AccessSearchSuccess(collection));
          }),
          catchError(error => {
            ctx.dispatch(new AccessSearchFail());
            throw error;
          }),
        );
    }
  }

  private getSearchStringToApply(ctx: StateContext<AccessStateModel>, action: AccessSearch): string {
    if (action.search === null || action.search === undefined) {
      return this.getDefaultSearchStringIfNotDefined(ctx);

    }
    return action.search;
  }

  private getDefaultSearchStringIfNotDefined(ctx: StateContext<AccessStateModel>): string {
    if (ctx.getState().search === null || ctx.getState().search === undefined) {
      return this.DEFAULT_SEARCH;
    }
    return ctx.getState().search;
  }

  @Action(AccessSearchSuccess)
  searchDipSuccess(ctx: StateContext<AccessStateModel>, action: AccessSearchSuccess) {
    const queryParameters = CrudHelperUtil.updateQueryParameters(ctx, action.list);

    ctx.patchState({
      list: action.list === null || action.list === undefined ? [] : action.list._data,
      total: action.list === null || action.list === undefined ? 0 : action.list._page.totalItems,
      isLoading: false,
      queryParameters,
    });
  }

  @Action(AccessSearchFail)
  searchDipFail(ctx: StateContext<AccessStateModel>) {
    ctx.patchState({
      isLoading: false,
    });
  }
}
