import {ChangeDetectionStrategy, Component, OnInit} from "@angular/core";
import {AccessChangeQueryParameters, AccessSearch} from "@app/access/access.action";
import {AccessStateModel} from "@app/access/access.state";
import {ArchiveModel} from "@app/access/models/archive.model";
import {OrganizationalUnitsModel} from "@app/generated-api";
import {BaseDirective} from "@app/shared/directives/base.directive";
import {FieldTypeEnum} from "@app/shared/enums/field-type.enum";
import {StateEnum} from "@app/shared/enums/state.enum";
import {ArchiveMetadataModel} from "@app/shared/models/business/archive-metadata.model";
import {DataTableColumnsModel} from "@app/shared/models/data-table-columns.model";
import {QueryParametersModel} from "@app/shared/models/query-parameters.model";
import {OrganizationalUnitStateModel} from "@app/shared/organizational-unit.state";
import {MetadataUtil} from "@app/shared/utils/metadata.util";
import {TRANSLATE} from "@app/shared/utils/translate.util";
import {TranslateService} from "@ngx-translate/core";
import {Select, Store} from "@ngxs/store";
import {BehaviorSubject, combineLatest, Observable} from "rxjs";
import {filter} from "rxjs/internal/operators/filter";
import {tap} from "rxjs/internal/operators/tap";


@Component({
  selector: "dlcm-access",
  templateUrl: "./access.view.html",
  styleUrls: ["./access.view.scss"],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class AccessView extends BaseDirective implements OnInit {
  @Select((state) => (state[StateEnum.access] as AccessStateModel).search) search$: Observable<boolean>;
  @Select((state) => (state[StateEnum.access] as AccessStateModel).isLoading) isLoading$: Observable<boolean>;
  @Select((state) => (state[StateEnum.access] as AccessStateModel).list) list$: Observable<ArchiveMetadataModel[]>;
  @Select((state) => (state[StateEnum.access] as AccessStateModel).queryParameters) queryParameters$: Observable<ArchiveMetadataModel[]>;
  @Select((state) => (state[StateEnum.organizationalUnit] as OrganizationalUnitStateModel).list) listOrganizationalUnit$: Observable<OrganizationalUnitsModel[]>;
  @Select((state) => (state[StateEnum.organizationalUnit] as OrganizationalUnitStateModel).isLoading) isLoadingOrgUnit$: Observable<boolean>;

  listAdapted$: Observable<any[]>;

  columns: DataTableColumnsModel[];

  listAdaptedSubject: BehaviorSubject<any>;

  constructor(private store: Store,
              private translate: TranslateService) {
    super();
    this.listAdaptedSubject = new BehaviorSubject([]);
    this.listAdapted$ = this.listAdaptedSubject.asObservable();
  }

  ngOnInit() {
    this.search(null);

    // TODO Use typeof to get interface propertie name
    this.columns = [
      {field: "organizationalUnit", header: TRANSLATE("access.table.header.organizationalUnit"), type: FieldTypeEnum.string, order: 0},
      {field: "title", header: TRANSLATE("access.table.header.title"), type: FieldTypeEnum.string, order: 0},
      {field: "yearPublicationDate", header: TRANSLATE("access.table.header.yearPublicationDate"), type: FieldTypeEnum.number, order: 0},
      {field: "accessLevel", header: TRANSLATE("access.table.header.accessLevel"), type: FieldTypeEnum.string, order: 0},
    ];

    this.addToSubscription(
      combineLatest(this.list$, this.listOrganizationalUnit$)
        .pipe(
          filter(([list, listOrgUnits]) => list !== null && list !== undefined && listOrgUnits !== null && listOrgUnits !== undefined),
          tap(([listArchives, listOrgUnits]: [ArchiveMetadataModel[], OrganizationalUnitsModel[]]) => {
            const newList: any[] = [];
            listArchives.forEach((archive: ArchiveMetadataModel) => {
              newList.push({
                organizationalUnit: this.getOrganizationalUnitNameFromId(
                  MetadataUtil.getOrganizationalUnitResId(archive.metadata),
                  listOrgUnits,
                ),
                title: MetadataUtil.getTitle(archive.metadata),
                yearPublicationDate: MetadataUtil.getPublicationYear(archive.metadata),
                accessLevel: MetadataUtil.getAccessLevel(archive.metadata),
              } as ArchiveModel);
            });
            this.listAdaptedSubject.next(newList);
          }),
        ).subscribe(),
    );
  }

  getOrganizationalUnitNameFromId(resId: string, listOrgUnits: OrganizationalUnitsModel[]): string {
    const orgUnits = listOrgUnits.find(o => o.resId === resId);
    if (orgUnits) {
      return orgUnits.name;
    }
    return this.translate.instant("access.organizationalUnitNoMoreExist");
  }

  search(searchTerm: string) {
    this.store.dispatch(new AccessSearch(searchTerm));
  }

  onQueryParametersEvent($event: QueryParametersModel) {
    this.store.dispatch(new AccessChangeQueryParameters($event));
  }

  showDetail($event: any) {

  }


}
