import {ChangeAppLanguage, Login, LoginFail, LoginSuccess, Logout, SetDefaultLanguage, SetMomentLocal} from "@app/app.action";
import {authConfig} from "@app/auth.config";
import {BaseStateModel} from "@app/base.state";
import {OAuthService} from "@app/oauth/oauth-service";
import {LanguagesEnum} from "@app/shared/enums/languages.enum";
import {RoutesEnum} from "@app/shared/enums/routes.enum";
import {SessionStorageEnum} from "@app/shared/enums/session-storage.enum";
import {StateEnum} from "@app/shared/enums/state.enum";
import {UserPreferencesUtil} from "@app/shared/utils/user-preferences.util";
import {TranslateService} from "@ngx-translate/core";
import {Navigate} from "@ngxs/router-plugin";
import {Action, InitState, State, StateContext, Store} from "@ngxs/store";
import * as moment from "moment";
import {tap} from "rxjs/operators";
import {environment} from "../environments/environment";

export interface AppStateModel extends BaseStateModel {
  appLanguage: LanguagesEnum;
  isLoggedIn: boolean;
}

@State<AppStateModel>({
  name: StateEnum.application,
  defaults: {
    appLanguage: UserPreferencesUtil.getPreferredLanguage(),
    isLoading: false,
    isLoggedIn: false,
  },
})
export class AppState {
  constructor(private store: Store,
              private translate: TranslateService,
              private oauthService: OAuthService) {
    this.oauthService.configure(authConfig);
    this.oauthService.setupAutomaticRefreshToken();
  }


  @Action(InitState)
  initApplication(ctx: StateContext<AppStateModel>) {
    ctx.dispatch([
      new Login(),
      new SetDefaultLanguage(),
      // new GetAllResources(), // TO FIX Don't work when dispatch
      new SetMomentLocal(),
    ]);
  }

  @Action(SetDefaultLanguage)
  setDefaultLanguage(ctx: StateContext<AppStateModel>) {
    // this language will be used as a fallback when a translation isn't found in the current language
    this.translate.setDefaultLang(environment.defaultLanguage);
  }

  @Action(SetMomentLocal)
  setMomentLocal(ctx: StateContext<AppStateModel>) {
    const navigatorLanguage = navigator.language;
    moment.locale(navigatorLanguage);
  }

  @Action(ChangeAppLanguage)
  changeAppLanguage(ctx: StateContext<AppStateModel>, action: ChangeAppLanguage) {
    // TOFIX this.translate.use(action.language); // Don't work properly here (need 2 action ChangeLanguage to be take in consideration...)

    ctx.patchState({
      appLanguage: action.language,
    });

    sessionStorage.setItem(SessionStorageEnum.language, action.language);
  }

  @Action(Login)
  login(ctx: StateContext<AppStateModel>) {
    return this.oauthService.tryLogin().pipe(
      tap(e => {
          if (e) {
            ctx.dispatch(new LoginSuccess());
          } else {
            ctx.dispatch(new LoginFail());
          }
        },
      ));
  }

  @Action(LoginSuccess)
  loginSuccess(ctx: StateContext<AppStateModel>) {
    ctx.patchState({
      isLoggedIn: true,
    });
  }


  @Action(LoginFail)
  loginFail(ctx: StateContext<AppStateModel>) {
    ctx.patchState({
      isLoggedIn: false,
    });
  }

  @Action(Logout)
  logout(ctx: StateContext<AppStateModel>) {
    this.oauthService.logOut();
    ctx.dispatch(new Navigate([RoutesEnum.access]));
    ctx.patchState({
      isLoggedIn: false,
    });
  }
}
