import {Injectable, NgZone, Optional} from "@angular/core";
import {HttpClient, HttpHeaders, HttpParams} from "@angular/common/http";
import {Observable, of, Subject, Subscription} from "rxjs";
import {delay, filter} from "rxjs/operators";
import {OAuthErrorEvent, OAuthEvent, OAuthInfoEvent, OAuthSuccessEvent} from "./events";
import {LoginOptions, OAuthStorage, TokenResponse} from "./types";
import {AuthConfig} from "./auth.config";

/**
 * Service for logging in and logging out with
 * OIDC and OAuth2. Supports code flow.
 */
@Injectable()
export class OAuthService extends AuthConfig {
  // extending AuthConfig ist just for LEGACY reasons
  // to not break existing code

  /**
   * Informs about events, like token_received or token_expires.
   * See the string enum EventType for a full list of events.
   */
  public events: Observable<OAuthEvent>;
  private refreshTokenPostMessageEventListener: EventListener;
  private eventsSubject: Subject<OAuthEvent> = new Subject<OAuthEvent>();
  private _storage: OAuthStorage;
  private accessTokenTimeoutSubscription: Subscription;
  private readonly RESPONSE_TYPE = "code";

  constructor(private ngZone: NgZone,
              private http: HttpClient,
              @Optional() storage: OAuthStorage,
              @Optional() private config: AuthConfig) {
    super();
    this.events = this.eventsSubject.asObservable();

    if (config) {
      this.configure(config);
    }

    try {
      if (storage) {
        this.setStorage(storage);
      } else if (typeof sessionStorage !== "undefined") {
        this.setStorage(sessionStorage);
      }
    } catch (e) {
      console.error("cannot access sessionStorage. " +
        "Consider setting an own storage implementation using setStorage", e);
    }
    this.setupRefreshTimer();
  }

  /**
   * Use this method to configure the service
   * @param config the configuration
   */
  public configure(config: AuthConfig) {
    // For the sake of downward compatibility with
    // original configuration API
    Object.assign(this, new AuthConfig(), config);

    this.config = Object.assign({} as AuthConfig, new AuthConfig(), config);
  }

  /**
   * Sets a custom storage used to store the received
   * tokens on client side. By default, the browser's
   * sessionStorage is used.
   *
   * @param storage storage service
   */
  public setStorage(storage: OAuthStorage): void {
    this._storage = storage;
  }

  /**
   * Refreshes the token using a refresh_token.
   * This does not work for implicit flow, b/c
   * there is no refresh_token in this flow.
   */
  public refreshToken() {
    let params = new HttpParams()
      .set("grant_type", "refresh_token")
      .set("refresh_token", this._storage.getItem("refresh_token"))
      .set("scope", this.scope);
    if (this.dummyClientSecret) {
      params = params.set("client_secret", this.dummyClientSecret);
    }
    this.fetchToken(params);
  }

  /**
   * Setup an automatic refresh token call when the access token expires
   */
  public setupAutomaticRefreshToken() {
    this.events.pipe(filter(e => e.type === "token_expires")).subscribe(() => {
      this.setupRefreshTokenEventListener();
      this.refreshToken();
    });

    this.restartRefreshTimerIfStillLoggedIn();
  }

  /**
   * Starts the authorization code flow and redirects to user to
   * the auth servers login url.
   */
  public initAuthorizationCodeFlow(): void {

    if (!this.validateUrlForHttps(this.loginUrl)) {
      throw new Error("loginUrl must use Http. Also check property requireHttps.");
    }

    this.createLoginUrl("", "", null, false, {})
      .subscribe(url => {
        location.href = url;
      }, error => {
        console.error("Error in initAuthorizationCodeFlow");
        console.error(error);
      });
  }

  /**
   * Checks whether there are tokens in the hash fragment
   * as a result of the implicit flow. These tokens are
   * parsed, validated and used to sign the user in to the
   * current client.
   *
   * @param options Optinal options.
   */
  public tryLogin(options: LoginOptions = null): Observable<boolean> {
    if (!this.requestAccessToken && !this.oidc) {
      this.debug("Either requestAccessToken or oidc or both must be true.");
      return of(false);
    }

    if (this.getAccessToken()) {
      return of(true);
    } else if (new Date(this.getAccessTokenExpiration()) >= new Date()) {
      return of(true);
    } else if (window.location.search && (window.location.search.startsWith("?code=") || window.location.search.includes("&code="))) {
      const parameter = window.location.search.split("?")[1].split("&");
      const codeParam = parameter.filter(param => param.includes("code="));
      const code = codeParam.length ? codeParam[0].split("code=")[1] : undefined;
      if (code) {
        try {
          this.getTokenFromCode(code);
          return of(true);
        } catch (e) {
          return of(false);
        }
      }
    }
    return of(false);
  }

  /**
   * Returns the received claims about the user.
   */
  public getIdentityClaims(): any {
    const claims = this._storage.getItem("id_token_claims_obj");
    if (!claims) {
      return null;
    }
    return JSON.parse(claims);
  }

  /**
   * Returns the current access_token.
   */
  public getAccessToken(): string {
    return this._storage.getItem("access_token");
  }

  /**
   * Returns the current refresh_token
   */
  public getRefreshToken(): string {
    return this._storage.getItem("refresh_token");
  }

  /**
   * Returns the expiration date of the access_token
   * as milliseconds since 1970.
   */
  public getAccessTokenExpiration(): number {
    if (!this._storage.getItem("expires_at")) {
      return null;
    }
    return parseInt(this._storage.getItem("expires_at"), 10);
  }

  /**
   * Checkes, whether there is a valid access_token.
   */
  public hasValidAccessToken(): boolean {
    if (this.getAccessToken()) {
      const expiresAt = this._storage.getItem("expires_at");
      const now = new Date();

      return !(expiresAt && parseInt(expiresAt, 10) < now.getTime());
    }
    return false;
  }

  /**
   * Removes all tokens and logs the user out.
   * If a logout url is configured, the user is
   * redirected to it.
   * @param noRedirectToLogoutUrl boolean to redirect or not
   */
  public logOut(noRedirectToLogoutUrl = false): void {
    this.clearStorage();

    this.eventsSubject.next(new OAuthInfoEvent("logout"));

    if (!this.logoutUrl) {
      return;
    }
    if (noRedirectToLogoutUrl) {
      return;
    }
    let logoutUrl: string;

    if (!this.validateUrlForHttps(this.logoutUrl)) {
      throw new Error("logoutUrl must use Http. Also check property requireHttps.");
    }

    // For backward compatibility
    if (this.logoutUrl.indexOf("{{") > -1) {
      logoutUrl = this.logoutUrl
        .replace(/\{\{client_id\}\}/, this.clientId);
    } else {
      logoutUrl =
        this.logoutUrl +
        (this.logoutUrl.indexOf("?") > -1 ? "&" : "?") +
        "id_token_hint=" +
        encodeURIComponent("") +
        "&post_logout_redirect_uri=" +
        encodeURIComponent(this.postLogoutRedirectUri || this.redirectUri);
    }
    location.href = logoutUrl;
  }

  /**
   * @ignore
   */
  public createAndSaveNonce(): string {
    const nonce = this.createNonce();
    this._storage.setItem("nonce", nonce);
    return nonce;
  }

  protected createNonce(): string {

    if (this.rngUrl) {
      throw new Error("createNonce with rng-web-api has not been implemented so far");
    } else {
      let text = "";
      const possible = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789";

      for (let i = 0; i < 40; i++) {
        text += possible.charAt(Math.floor(Math.random() * possible.length));
      }

      return text;
    }
  }

  private setupRefreshTokenEventListener(): void {
    this.removeRefreshTokenEventListener();

    this.refreshTokenPostMessageEventListener = () => {
      this.refreshToken();
    };
    window.addEventListener("message", this.refreshTokenPostMessageEventListener);
  }

  private removeRefreshTokenEventListener(): void {
    if (this.refreshTokenPostMessageEventListener) {
      window.removeEventListener("message", this.refreshTokenPostMessageEventListener);
      this.refreshTokenPostMessageEventListener = null;
    }
  }

  private restartRefreshTimerIfStillLoggedIn(): void {
    this.setupExpirationTimers();
  }

  private createLoginUrl(state = "",
                         loginHint = "",
                         customRedirectUri = "",
                         noPrompt = false,
                         params: object = {}): Observable<string> {
    const that = this;

    let redirectUri: string;

    if (customRedirectUri) {
      redirectUri = customRedirectUri;
    } else {
      redirectUri = this.redirectUri;
    }

    let nonce = null;
    if (!this.disableNonceCheck) {
      nonce = this.createAndSaveNonce();
      if (state) {
        state = nonce + this.config.nonceStateSeparator + state;
      } else {
        state = nonce;
      }
    }

    if (!this.requestAccessToken && !this.oidc) {
      throw new Error(
        "Either requestAccessToken or oidc or both must be true"
      );
    }

    this.responseType = this.RESPONSE_TYPE;

    const separationChar = that.loginUrl.indexOf("?") > -1 ? "&" : "?";

    const scope = that.scope;

    let url =
      that.loginUrl +
      separationChar +
      "response_type=" +
      encodeURIComponent(that.responseType) +
      "&client_id=" +
      encodeURIComponent(that.clientId) +
      "&state=" +
      encodeURIComponent(state) +
      "&redirect_uri=" +
      encodeURIComponent(redirectUri) +
      "&scope=" +
      encodeURIComponent(scope);

    if (loginHint) {
      url += "&login_hint=" + encodeURIComponent(loginHint);
    }

    if (that.resource) {
      url += "&resource=" + encodeURIComponent(that.resource);
    }

    if (nonce && this.oidc) {
      url += "&nonce=" + encodeURIComponent(nonce);
    }

    if (noPrompt) {
      url += "&prompt=none";
    }

    for (const key of Object.keys(params)) {
      url +=
        "&" + encodeURIComponent(key) + "=" + encodeURIComponent(params[key]);
    }

    if (this.customQueryParams) {
      for (const key of Object.getOwnPropertyNames(this.customQueryParams)) {
        url +=
          "&" + key + "=" + encodeURIComponent(this.customQueryParams[key]);
      }
    }

    return of(url);
  }

  private storeAccessTokenResponse(accessToken: string, refreshToken: string, expiresIn: number, grantedScopes: string): void {
    this._storage.setItem("access_token", accessToken);
    if (grantedScopes) {
      this._storage.setItem("granted_scopes", JSON.stringify(grantedScopes.split("+")));
    }
    this._storage.setItem("access_token_stored_at", "" + Date.now());
    if (expiresIn) {
      const expiresInMilliSeconds = expiresIn * 1000;
      const now = new Date();
      const expiresAt = now.getTime() + expiresInMilliSeconds;
      this._storage.setItem("expires_at", "" + expiresAt);
    }

    if (refreshToken) {
      this._storage.setItem("refresh_token", refreshToken);
    }
  }

  private debug(...args): void {
    if (this.showDebugInformation) {
      console.log.apply(console, args);
    }
  }

  private validateUrlForHttps(url: string): boolean {
    if (!url) {
      return true;
    }

    const lcUrl = url.toLowerCase();

    if (this.requireHttps === false) {
      return true;
    }

    if ((lcUrl.match(/^http:\/\/localhost($|[:\/])/)
      || lcUrl.match(/^http:\/\/localhost($|[:\/])/))
      && this.requireHttps === "remoteOnly") {
      return true;
    }

    return lcUrl.startsWith("https://");
  }

  private setupRefreshTimer(): void {
    if (typeof window === "undefined") {
      this.debug("timer not supported on this plattform");
      return;
    }

    this.events.pipe(filter(e => e.type === "token_received"))
      .subscribe(() => {
        this.clearAccessTokenTimer();
        this.setupExpirationTimers();
      });
  }

  private setupExpirationTimers(): void {
    if (this.hasValidAccessToken()) {
      this.setupAccessTokenTimer();
    }
  }

  private setupAccessTokenTimer(): void {
    const expiration = this.getAccessTokenExpiration();
    const storedAt = this.getAccessTokenStoredAt();
    const timeout = this.calcTimeout(storedAt, expiration);

    this.ngZone.runOutsideAngular(() => {
      this.accessTokenTimeoutSubscription = of(new OAuthInfoEvent("token_expires", "access_token"))
        .pipe(delay(timeout))
        .subscribe(e => {
          this.ngZone.run(() => {
            this.eventsSubject.next(e);
          });
        });
    });
  }

  private clearAccessTokenTimer(): void {
    if (this.accessTokenTimeoutSubscription) {
      this.accessTokenTimeoutSubscription.unsubscribe();
    }
  }

  private calcTimeout(storedAt: number, expiration: number): number {
    return (expiration - storedAt) * this.timeoutFactor;
  }

  /**
   * Get token using an intermediate code. Works for the Authorization Code flow.
   */
  private getTokenFromCode(code: string) {
    const params = new HttpParams()
      .set("grant_type", "authorization_code")
      .set("code", code)
      .set("redirect_uri", this.redirectUri);
    this.fetchToken(params);
  }

  private fetchToken(params: HttpParams) {
    if (!this.validateUrlForHttps(this.tokenEndpoint)) {
      throw new Error("tokenEndpoint must use Http. Also check property requireHttps.");
    }

    params = params.set("client_id", this.clientId);

    if (this.customQueryParams) {
      for (const key of Object.getOwnPropertyNames(this.customQueryParams)) {
        params = params.set(key, this.customQueryParams[key]);
      }
    }

    const authData = window.btoa(this.config.clientId + ":" + this.config.dummyClientSecret);
    const headers = new HttpHeaders()
      .set("Content-Type", "application/x-www-form-urlencoded")
      .set("Authorization", "Basic " + authData);

    this.http.post<TokenResponse>(this.tokenEndpoint, params, {headers})
      .subscribe((tokenResp) => {
          this.debug("refresh tokenResponse", tokenResp);
          this.storeAccessTokenResponse(tokenResp.access_token, tokenResp.refresh_token, tokenResp.expires_in, tokenResp.scope);
          this.eventsSubject.next(new OAuthSuccessEvent("token_received"));
          this.eventsSubject.next(new OAuthSuccessEvent("token_refreshed"));
        },
        (err) => {
          console.error("Error getting token", err);
          this.eventsSubject.next(new OAuthErrorEvent("token_refresh_error", err));
          throw err;
        }
      );
  }

  private getAccessTokenStoredAt(): number {
    return parseInt(this._storage.getItem("access_token_stored_at"), 10);
  }

  private clearStorage() {
    this._storage.removeItem("access_token");
    this._storage.removeItem("id_token");
    this._storage.removeItem("refresh_token");
    this._storage.removeItem("nonce");
    this._storage.removeItem("expires_at");
    this._storage.removeItem("id_token_claims_obj");
    this._storage.removeItem("id_token_expires_at");
    this._storage.removeItem("id_token_stored_at");
    this._storage.removeItem("access_token_stored_at");
  }

}
