import {EventEmitter, Input, OnInit, Output} from "@angular/core";
import {AbstractControl, FormGroup, Validators} from "@angular/forms";
import {BaseDirective} from "@app/shared/directives/base.directive";

export abstract class AbstractFormComponent<T> extends BaseDirective implements OnInit {
  public form: FormGroup;

  @Input()
  model: T;

  @Input()
  readonly: boolean;

  @Output()
  submitEvent: EventEmitter<T>;

  required = Validators.required;

  protected constructor() {
    super();
    this.submitEvent = new EventEmitter<T>();
  }

  ngOnInit() {
    if (this.model) {
      this.bindFormTo(this.model);
    } else {
      this.initNewForm();
    }
    this.disableIfReadonly();
  }

  private disableIfReadonly() {
    if (this.readonly) {
      this.form.disable();
    }
  }

  protected abstract initNewForm();

  protected abstract bindFormTo(model: T);

  public onSubmit() {
    let model = this.form.value as T;
    model = this.treatmentBeforeSubmit(model);
    this.addResId(model);
    this.submitEvent.emit(model);
  }

  protected abstract treatmentBeforeSubmit(model: T): T;

  private addResId(model: T) {
    const isUpdate = this.model !== null && this.model !== undefined;
    if (isUpdate) {
      model["resId"] = this.model["resId"];
    }
  }

  getFormControl(key: string): AbstractControl {
    return this.form.get(key);
  }

  isRequired(key: string): boolean {
    const errors = this.getFormControl(key).errors;
    if (errors == null) {
      return false;
    }
    return errors.required === true ? true : false;
  }
}
