import {ChangeDetectionStrategy, Component, EventEmitter, OnInit, Output} from "@angular/core";
import {ActivatedRoute, NavigationEnd, Router} from "@angular/router";
import {BaseDirective} from "@app/shared/directives/base.directive";
import {BreadcrumbModel} from "@app/shared/models/breadcrumb.model";
import {TRANSLATE} from "@app/shared/utils/translate.util";
import {TranslateService} from "@ngx-translate/core";
import {Observable} from "rxjs";
import {distinctUntilChanged} from "rxjs/internal/operators/distinctUntilChanged";
import {filter, map} from "rxjs/operators";

@Component({
  selector: "dlcm-breadcrumb",
  templateUrl: "./breadcrumb.component.html",
  styleUrls: ["./breadcrumb.component.scss"],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class BreadcrumbComponent extends BaseDirective implements OnInit {
  breadcrumbs$: Observable<BreadcrumbModel[]>;

  @Output()
  navigateEvent: EventEmitter<string>;

  private readonly BREADCRUMB_KEY: string = "breadcrumb";
  private readonly BREADCRUMB_TRANSLATE_HOME: string = TRANSLATE("breadcrumb.home");

  constructor(private translate: TranslateService,
              private router: Router,
              private activatedRoute: ActivatedRoute) {
    super();
    this.navigateEvent = new EventEmitter<string>();
  }

  ngOnInit() {
    this.breadcrumbs$ = this.router.events
      .pipe(
        filter(event => event instanceof NavigationEnd),
        distinctUntilChanged(),
        map(event => this.buildBreadCrumb(this.activatedRoute.root)),
      );
  }

  buildBreadCrumb(route: ActivatedRoute,
                  url: string = "",
                  breadcrumbs: Array<BreadcrumbModel> = []): Array<BreadcrumbModel> {
    const label = this.getLabel(route);
    const path = this.getPath(route);

    const fullUrl = `${url}${path}/`;

    const breadcrumb: BreadcrumbModel = {
      label,
      url: fullUrl,
    } as BreadcrumbModel;

    const newBreadcrumbs: BreadcrumbModel[] = [...breadcrumbs, breadcrumb];
    if (route.firstChild) {
      return this.buildBreadCrumb(route.firstChild, fullUrl, newBreadcrumbs);
    }
    return newBreadcrumbs;
  }

  private getLabel(route: ActivatedRoute): string {
    if (route.routeConfig) {
      const routeConfig = route.routeConfig;
      if (routeConfig.data == null || routeConfig.data[this.BREADCRUMB_KEY] == null) {
        throw Error(`Please define the data '${this.BREADCRUMB_KEY}', for the following route ${route.toString()}`);
      }
      return route.routeConfig.data[this.BREADCRUMB_KEY];
    }

    return this.BREADCRUMB_TRANSLATE_HOME;
  }

  private getPath(route: ActivatedRoute) {
    if (route.routeConfig) {
      let path = route.routeConfig.path;
      path = this.insertParametersInPath(route, path);
      return path;
    }
    return "";
  }

  private insertParametersInPath(route: ActivatedRoute, path) {
    const params = route.snapshot.params;
    for (const key in params) {
      path = path.replace(`:${key}`, params[key]);
    }
    return path;
  }

  navigate(url: string): void {
    this.navigateEvent.emit(url);
  }
}
