import {
  ChangeDetectionStrategy,
  Component,
  EventEmitter,
  Input,
  OnChanges,
  Output,
  SimpleChanges,
  ViewChild,
  ViewEncapsulation,
} from "@angular/core";
import {PaginatorComponent} from "@app/shared/components/paginator/paginator.component";
import {FieldTypeEnum} from "@app/shared/enums/field-type.enum";
import {OrderEnum} from "@app/shared/enums/order.enum";
import {DataTableColumnsModel} from "@app/shared/models/data-table-columns.model";
import {PagingModel} from "@app/shared/models/paging.model";
import {QueryParametersModel} from "@app/shared/models/query-parameters.model";
import {SortModel} from "@app/shared/models/sort.model";
import {MapUtil} from "@app/shared/utils/map.util";
import {ObjectUtil} from "@app/shared/utils/object.util";
import _ from "lodash";
import * as moment from "moment";
import {LazyLoadEvent} from "primeng/api";

@Component({
  selector: "dlcm-data-table",
  templateUrl: "./data-table.component.html",
  styleUrls: ["./data-table.component.scss"],
  encapsulation: ViewEncapsulation.None,
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class DataTableComponent<T> implements OnChanges {
  @Input()
  datas: T[];

  @Input()
  queryParameters: QueryParametersModel;

  @Input()
  columns: DataTableColumnsModel[];

  @Output()
  selectEvent: EventEmitter<T>;

  @Output()
  queryParametersEvent: EventEmitter<QueryParametersModel>;

  @ViewChild("paginator")
  paginator: PaginatorComponent;

  isDatasPresent: boolean;

  fieldTypeString: FieldTypeEnum = FieldTypeEnum.string;

  private readonly SEPARATOR = ".";
  private readonly FORMAT_TIME_ONLY = "LTS";
  private readonly FORMAT_DATE_ONLY = "L";

  constructor() {
    this.selectEvent = new EventEmitter<T>();
    this.queryParametersEvent = new EventEmitter<QueryParametersModel>();
  }

  ngOnChanges(changes: SimpleChanges): void {
    this.computeIsDatasPresent();
  }

  private computeIsDatasPresent() {
    this.isDatasPresent = this.datas.length > 0;
  }

  showDetail(row: T) {
    this.selectEvent.emit(row);
  }

  getCellData(row: any, column: DataTableColumnsModel): any {
    const data = this.getData(row, column);
    return this.applyTypeToData(data, column.type);
  }

  private getData(row: any, column: DataTableColumnsModel): any {
    const field = column.field;
    if (field.includes(this.SEPARATOR)) {
      return this.getNestedData(column, row);
    }
    return row[field];
  }

  private getNestedData(column: DataTableColumnsModel, row: any) {
    const nestedProperties: string[] = column.field.split(this.SEPARATOR);
    return row[_.last(nestedProperties)];
  }

  private applyTypeToData(data: any, type: FieldTypeEnum) {
    if (type === FieldTypeEnum.date) {
      return moment(data).format(this.FORMAT_DATE_ONLY);
    }
    if (type === FieldTypeEnum.datetime) {
      return moment(data).format(this.FORMAT_DATE_ONLY) + " " + moment(data).format(this.FORMAT_TIME_ONLY);
    }
    return data;
  }

  backendSort($event: LazyLoadEvent): void {
    if ($event.sortField === undefined) {
      return;
    }
    const sort = this.adaptLazyLoadEventToSortModel($event);
    const queryParameters = ObjectUtil.clone(this.queryParameters);
    queryParameters.sort = sort;
    this.queryParametersEvent.emit(queryParameters);
  }

  private adaptLazyLoadEventToSortModel($event: LazyLoadEvent): SortModel {
    return {
      field: $event.sortField,
      order: $event.sortOrder === -1 ? OrderEnum.descending : OrderEnum.ascending,
    } as SortModel;
  }

  pageEvent($event: PagingModel) {
    const queryParameters = ObjectUtil.clone(this.queryParameters);
    queryParameters.paging = $event;
    this.queryParametersEvent.emit(queryParameters);
  }

  onValueChange(col: DataTableColumnsModel, value: string) {
    const queryParameters = ObjectUtil.clone(this.queryParameters);
    if (value === "") {
      queryParameters.search.searchItems.delete(col.field);
    } else {
      MapUtil.addOrUpdate(queryParameters.search.searchItems, col.field, value);
    }
    this.queryParametersEvent.emit(queryParameters);
  }

  getValue(field: string) {
    const value = this.queryParameters.search.searchItems.get(field);
    if (value === null || value === undefined || value === "") {
      return null;
    }
    return value;
  }
}
