import {BaseStateModel} from "@app/base.state";
import {
  ChangeQueryParametersCrud,
  CreateCrud,
  CreateFailCrud,
  CreateSuccessCrud,
  DeleteCrud,
  DeleteFailCrud,
  DeleteSuccessCrud,
  GetAllCrud,
  GetAllFailCrud,
  GetAllSuccessCrud,
  GetByIdCrud,
  GetByIdFailCrud,
  GetByIdSuccessCrud,
  UpdateCrud,
  UpdateFailCrud,
  UpdateSuccessCrud,
} from "@app/shared/crud.action";
import {StateEnum} from "@app/shared/enums/state.enum";
import {CollectionTypedModel} from "@app/shared/models/collection-typed.model";
import {PagingModel} from "@app/shared/models/paging.model";
import {QueryParametersModel} from "@app/shared/models/query-parameters.model";
import {ApiService} from "@app/shared/services/api.service";
import {CrudActionUtil} from "@app/shared/utils/crud-action.util";
import {CrudHelperUtil} from "@app/shared/utils/crud-helper.util";
import {CrudRouteUtil} from "@app/shared/utils/crud-route.util";
import {ObjectUtil} from "@app/shared/utils/object.util";
import {Navigate} from "@ngxs/router-plugin";
import {StateContext} from "@ngxs/store";
import {Observable} from "rxjs";
import {catchError} from "rxjs/internal/operators/catchError";
import {tap} from "rxjs/operators";

export interface CrudStateModel<T> extends BaseStateModel {
  total: number;
  list: T[];
  current: T;
  queryParameters: QueryParametersModel;
}

export abstract class CrudState<T> {
  protected abstract urlResource: string;
  protected abstract state: StateEnum;

  protected constructor(protected apiService: ApiService) {
  }

  changeQueryParameters(ctx: StateContext<CrudStateModel<T>>, action: ChangeQueryParametersCrud): void {
    ctx.patchState({
      queryParameters: action.queryParameters,
    });
    ctx.dispatch(CrudActionUtil.getAll(this.state));
  }

  getAll(ctx: StateContext<CrudStateModel<T>>, action: GetAllCrud): Observable<CollectionTypedModel<T>> {
    ctx.patchState({
      isLoading: true,
    });

    const queryParameters = CrudHelperUtil.getQueryParametersToApply(action.queryParameters, ctx);

    return this.apiService.get<T>(this.urlResource, queryParameters)
      .pipe(
        tap((collection: CollectionTypedModel<T>) => {
          ctx.dispatch(CrudActionUtil.getAllSuccess(this.state, collection));
        }),
        catchError(error => {
          ctx.dispatch(CrudActionUtil.getAllFail(this.state));
          throw error;
        }),
      );
  }

  getAllSuccess(ctx: StateContext<CrudStateModel<T>>, action: GetAllSuccessCrud<T>): void {
    const queryParameters = CrudHelperUtil.updateQueryParameters(ctx, action.list);

    ctx.patchState({
      list: action.list._data,
      total: action.list._page.totalItems,
      isLoading: false,
      queryParameters,
    });
  }

  getAllFail(ctx: StateContext<CrudStateModel<T>>, action: GetAllFailCrud): void {
    ctx.patchState({
      isLoading: false,
    });
  }

  getById(ctx: StateContext<CrudStateModel<T>>, action: GetByIdCrud): Observable<T> {
    ctx.patchState({
      isLoading: true,
      current: null,
    });

    return this.apiService.getById<T>(this.urlResource, action.id)
      .pipe(
        tap((model: T) => {
          ctx.dispatch(CrudActionUtil.getByIdSuccess(this.state, model));
        }),
        catchError(error => {
          ctx.dispatch(CrudActionUtil.getByIdFail(this.state));
          throw error;
        }),
      );
  }

  getByIdSuccess(ctx: StateContext<CrudStateModel<T>>, action: GetByIdSuccessCrud<T>): void {
    ctx.patchState({
      current: action.model,
      isLoading: false,
    });
  }

  getByIdFail(ctx: StateContext<CrudStateModel<T>>, action: GetByIdFailCrud): void {
    ctx.patchState({
      isLoading: false,
    });
  }

  create(ctx: StateContext<CrudStateModel<T>>, action: CreateCrud<T>): Observable<T> {
    ctx.patchState({
      isLoading: true,
    });

    return this.apiService.post<T>(this.urlResource, action.model)
      .pipe(
        tap(() => {
          ctx.dispatch(CrudActionUtil.createSuccess(this.state));
        }),
        catchError(error => {
          ctx.dispatch(CrudActionUtil.createFail(this.state));
          throw error;
        }),
      );
  }

  createSuccess(ctx: StateContext<CrudStateModel<T>>, action: CreateSuccessCrud): void {
    ctx.patchState({
      isLoading: false,
    });

    ctx.dispatch(new Navigate([CrudRouteUtil.getRootRoute(this.state)]));
  }

  createFail(ctx: StateContext<CrudStateModel<T>>, action: CreateFailCrud): void {
    ctx.patchState({
      isLoading: false,
    });
  }

  update(ctx: StateContext<CrudStateModel<T>>, action: UpdateCrud<T>): Observable<T> {
    ctx.patchState({
      isLoading: true,
    });

    return this.apiService.patchById<T>(this.urlResource, action.model["resId"], action.model)
      .pipe(
        tap(() => {
          ctx.dispatch(CrudActionUtil.updateSuccess(this.state));
        }),
        catchError(error => {
          ctx.dispatch(CrudActionUtil.updateFail(this.state));
          throw error;
        }),
      );
  }

  updateSuccess(ctx: StateContext<CrudStateModel<T>>, action: UpdateSuccessCrud): void {
    ctx.patchState({
      isLoading: false,
    });

    ctx.dispatch(new Navigate([CrudRouteUtil.getRootRoute(this.state)]));
  }

  updateFail(ctx: StateContext<CrudStateModel<T>>, action: UpdateFailCrud): void {
    ctx.patchState({
      isLoading: false,
    });
  }

  delete(ctx: StateContext<CrudStateModel<T>>, action: DeleteCrud): Observable<T> {
    ctx.patchState({
      isLoading: true,
    });

    return this.apiService.deleteById<T>(this.urlResource, action.resId)
      .pipe(
        tap(() => {
          ctx.dispatch(CrudActionUtil.deleteSuccess(this.state));
        }),
        catchError(error => {
          ctx.dispatch(CrudActionUtil.deleteFail(this.state));
          throw error;
        }),
      );
  }

  deleteSuccess(ctx: StateContext<CrudStateModel<T>>, action: DeleteSuccessCrud): void {
    ctx.patchState({
      isLoading: false,
    });

    ctx.dispatch(new Navigate([CrudRouteUtil.getRootRoute(this.state)]));
  }

  deleteFail(ctx: StateContext<CrudStateModel<T>>, action: DeleteFailCrud): void {
    ctx.patchState({
      isLoading: false,
    });
  }


}
