import {HttpClient, HttpEvent, HttpHeaders, HttpParams, HttpResponse} from "@angular/common/http";
import {Inject, Injectable, Optional} from "@angular/core";
import {BASE_PATH, Configuration} from "@app/generated-api";
import {CustomHttpUrlEncodingCodec} from "@app/generated-api/encoder";
import {ResourceApiEnum} from "@app/shared/enums/api.enum";
import {CollectionTypedModel} from "@app/shared/models/collection-typed.model";
import {QueryParametersModel} from "@app/shared/models/query-parameters.model";
import {QueryParametersUtil} from "@app/shared/utils/queryParametersUtil";
import {Observable} from "rxjs";

@Injectable()
export class ApiService {
  protected basePath = "http://localhost";
  public defaultHeaders = new HttpHeaders();
  public configuration = new Configuration();

  constructor(protected httpClient: HttpClient, @Optional() @Inject(BASE_PATH) basePath: string, @Optional() configuration: Configuration) {
    if (configuration) {
      this.configuration = configuration;
      this.configuration.basePath = configuration.basePath || basePath || this.basePath;

    } else {
      this.configuration.basePath = basePath || this.basePath;
    }
  }

  /**
   * GET
   * To get a collection of resource
   * @param path The resource path
   * @param size The page size
   * @param page The current page number
   * @param sort To sort on a field. By default, the sort is ascending. desc option \&#39;[,desc]\&#39; permits to have descending order.
   * @param customParameters To apply n other query parameters.
   * @param observe set whether or not to return the data Observable as the body, response or events. defaults to returning the body.
   * @param reportProgress flag to report request and response progress.
   */
  public get<T>(path: ResourceApiEnum, queryParameters: QueryParametersModel, customParameters?: { [key: string]: string; }, observe?: 'body', reportProgress?: boolean): Observable<CollectionTypedModel<T>>;
  public get<T>(path: ResourceApiEnum, queryParameters: QueryParametersModel, customParameters?: { [key: string]: string; }, observe?: 'response', reportProgress?: boolean): Observable<HttpResponse<CollectionTypedModel<T>>>;
  public get<T>(path: ResourceApiEnum, queryParameters: QueryParametersModel, customParameters?: { [key: string]: string; }, observe?: 'events', reportProgress?: boolean): Observable<HttpEvent<CollectionTypedModel<T>>>;
  public get<T>(path: ResourceApiEnum, queryParameters?: QueryParametersModel, customParameters?: { [key: string]: string; }, observe: any = 'body', reportProgress: boolean = false): Observable<any> {
    const queryParametersHttp = this.getQueryParameters(queryParameters, customParameters);
    const headers = this.getHeaders();

    const result: Observable<HttpEvent<CollectionTypedModel<T>>>
      = this.httpClient.get<CollectionTypedModel<T>>(`${this.configuration.basePath}/${path}`,
      {
        params: queryParametersHttp,
        withCredentials: this.configuration.withCredentials,
        headers,
        observe,
        reportProgress,
      },
    );

    return result;
  }

  /**
   * GET BY ID
   * To get a specific resource
   * @param path The resource path
   * @param resId Resource Id (GUID)
   * @param observe set whether or not to return the data Observable as the body, response or events. defaults to returning the body.
   * @param reportProgress flag to report request and response progress.
   */
  public getById<T>(path: ResourceApiEnum, resId: string, observe?: "body", reportProgress?: boolean): Observable<T>;
  public getById<T>(path: ResourceApiEnum, resId: string, observe?: "response", reportProgress?: boolean): Observable<HttpResponse<T>>;
  public getById<T>(path: ResourceApiEnum, resId: string, observe?: "events", reportProgress?: boolean): Observable<HttpEvent<T>>;
  public getById<T>(path: ResourceApiEnum, resId: string, observe: any = "body", reportProgress: boolean = false): Observable<any> {
    if (resId === null || resId === undefined) {
      throw new Error(`Required parameter resId was null or undefined when calling ${path}GetById.`);
    }
    const headers = this.getHeaders();

    return this.httpClient.get<T>(`${this.configuration.basePath}/${path}/${encodeURIComponent(String(resId))}`,
      {
        withCredentials: this.configuration.withCredentials,
        headers,
        observe,
        reportProgress,
      },
    );
  }

  /**
   * DELETE
   * To delete an existing resource
   * @param path The resource path
   * @param resId Resource Id (GUID)
   * @param observe set whether or not to return the data Observable as the body, response or events. defaults to returning the body.
   * @param reportProgress flag to report request and response progress.
   */
  public deleteById<T>(path: ResourceApiEnum, resId: string, observe?: "body", reportProgress?: boolean): Observable<T>;
  public deleteById<T>(path: ResourceApiEnum, resId: string, observe?: "response", reportProgress?: boolean): Observable<HttpResponse<T>>;
  public deleteById<T>(path: ResourceApiEnum, resId: string, observe?: "events", reportProgress?: boolean): Observable<HttpEvent<T>>;
  public deleteById<T>(path: ResourceApiEnum, resId: string, observe: any = "body", reportProgress: boolean = false): Observable<any> {
    if (resId === null || resId === undefined) {
      throw new Error(`Required parameter resId was null or undefined when calling ${path}DeleteById.`);
    }
    const headers = this.getHeaders();

    return this.httpClient.delete<T>(`${this.configuration.basePath}/${path}/${encodeURIComponent(String(resId))}`,
      {
        withCredentials: this.configuration.withCredentials,
        headers,
        observe,
        reportProgress,
      },
    );
  }

  /**
   * PATCH
   * To edit a existing resource
   * @param path The resource path
   * @param resId Resource Id (GUID)
   * @param body The body to object to update
   * @param observe set whether or not to return the data Observable as the body, response or events. defaults to returning the body.
   * @param reportProgress flag to report request and response progress.
   */
  public patchById<T>(path: ResourceApiEnum, resId: string, body?: T, observe?: "body", reportProgress?: boolean): Observable<T>;
  public patchById<T>(path: ResourceApiEnum, resId: string, body?: T, observe?: "response", reportProgress?: boolean): Observable<HttpResponse<T>>;
  public patchById<T>(path: ResourceApiEnum, resId: string, body?: T, observe?: "events", reportProgress?: boolean): Observable<HttpEvent<T>>;
  public patchById<T>(path: ResourceApiEnum, resId: string, body?: T, observe: any = "body", reportProgress: boolean = false): Observable<any> {
    if (resId === null || resId === undefined) {
      throw new Error("Required parameter resId was null or undefined when calling accessDipPatchById.");
    }
    let headers = this.getHeaders();
    headers = this.addContentType(headers);

    return this.httpClient.patch<T>(`${this.configuration.basePath}/${path}/${encodeURIComponent(String(resId))}`,
      body,
      {
        withCredentials: this.configuration.withCredentials,
        headers,
        observe,
        reportProgress,
      },
    );
  }

  /**
   * POST
   * To create a new resource
   * @param path The resource path
   * @param body The body of object to create
   * @param observe set whether or not to return the data Observable as the body, response or events. defaults to returning the body.
   * @param reportProgress flag to report request and response progress.
   */
  public post<T>(path: ResourceApiEnum, body?: T, observe?: "body", reportProgress?: boolean): Observable<T>;
  public post<T>(path: ResourceApiEnum, body?: T, observe?: "response", reportProgress?: boolean): Observable<HttpResponse<T>>;
  public post<T>(path: ResourceApiEnum, body?: T, observe?: "events", reportProgress?: boolean): Observable<HttpEvent<T>>;
  public post<T>(path: ResourceApiEnum, body?: T, observe: any = "body", reportProgress: boolean = false): Observable<any> {
    let headers = this.getHeaders();
    headers = this.addContentType(headers);

    return this.httpClient.post<T>(`${this.configuration.basePath}/${path}`,
      body,
      {
        withCredentials: this.configuration.withCredentials,
        headers,
        observe,
        reportProgress,
      },
    );
  }

  private getQueryParameters(queryParameters: QueryParametersModel, customParameters: { [p: string]: string }) {
    let queryParametersHttp = new HttpParams({encoder: new CustomHttpUrlEncodingCodec()});
    if (queryParameters == null) {
      return queryParametersHttp;
    }

    const size = QueryParametersUtil.getPageSize(queryParameters);
    if (size !== undefined && size !== null) {
      queryParametersHttp = queryParametersHttp.set("size", size as any);
    }

    const page = QueryParametersUtil.getPageIndex(queryParameters);
    if (page !== undefined && page !== null) {
      queryParametersHttp = queryParametersHttp.set("page", page as any);
    }

    const sort = QueryParametersUtil.getSortString(queryParameters);
    if (sort !== undefined && sort !== null) {
      queryParametersHttp = queryParametersHttp.set("sort", sort as any);
    }

    const searchFilters = queryParameters.search;
    if (searchFilters !== undefined && searchFilters !== null) {
      searchFilters.searchItems.forEach((value: string, field: string) => {
        queryParametersHttp = queryParametersHttp.set(field, value as any);
      });
    }

    if (customParameters !== undefined && customParameters !== null) {
      for (const key in customParameters) {
        queryParametersHttp = queryParametersHttp.set(key, customParameters[key] as any);
      }
    }

    return queryParametersHttp;
  }

  private getHeaders(): HttpHeaders {
    let headers = this.defaultHeaders;

    // to determine the Accept header
    const httpHeaderAccepts: string[] = [
      "application/hal+json",
    ];
    const httpHeaderAcceptSelected: string | undefined = this.configuration.selectHeaderAccept(httpHeaderAccepts);
    if (httpHeaderAcceptSelected !== undefined) {
      headers = headers.set("Accept", httpHeaderAcceptSelected);
    }
    return headers;
  }

  private addContentType(headers) {
    const consumes: string[] = [
      "application/json",
    ];
    const httpContentTypeSelected: string | undefined = this.configuration.selectHeaderContentType(consumes);
    if (httpContentTypeSelected !== undefined) {
      headers = headers.set("Content-Type", httpContentTypeSelected);
    }
    return headers;
  }
}
